/*
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
        programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

/* System. */
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

/* Our own. */
#include "alphan.h"
#include "alphCRYPT.h"


/* Random select, haystack, pool, exclusive. */
int 
rselect(int limit, int *needle, int pinsz) 
{
	unsigned int *select;
	register int itra, itrb, itrc;
	unsigned int sem;
	unsigned int *hay;

	/* We can wave goodbye to srand(). Naaasty fucker... */
	
	if(!pinsz)
	{
		return rand()%limit;
	}
	
	hay = (unsigned int *) calloc(limit, sizeof(unsigned int));
	for(itra=0, itrc=0; itra<limit; itra++)
		hay[itrc++]=itra;
	
	select = (unsigned int *) calloc(limit-pinsz, sizeof(unsigned int));
	for(itra=limit-1, itrc=0; itra>=0; sem=0, itra--)
	{
		for(itrb=pinsz-1; itrb>=0; itrb--)
			if(hay[itra]==needle[itrb])
				sem++;
		if(!sem)
			{
				select[itrc++]=hay[itra];
			}
	}
	
	init_by_array(itrc);
	return select[genrand_int32()%itrc];
	
}

/* Basic frequency analyzer. */
char *
frq (char *block)
{

  unsigned int itra = 0, itrb = 0, itrd = 0;
  unsigned int *a26, curs = 0, ptr = 0, tmp = 0;
  char *smash;
  char lclatin[] = {
    'a', 'b', 'c', 'd', 'e',
    'f', 'g', 'h', 'i', 'j',
    'k', 'l', 'm', 'n', 'o',
    'p', 'q', 'r', 's', 't',
    'u', 'v', 'w', 'x', 'y',
    'z'
  };
  char fen[] = {
    'e', 't', 'a', 'o', 'i',
    'n', 's', 'r', 'h', 'l',
    'd', 'c', 'u', 'm', 'f',
    'p', 'g', 'w', 'y', 'b',
    'v', 'k', 'x', 'j', 'q',
    'z'
  };
  
  a26 = (int *) calloc(25, sizeof(int));
  
  smash = (char *) calloc (strlen (block), sizeof (char));
  strcpy (smash, block);
  for (itra = 0; itra < strlen (block); itra++)
    {
      if (isupper (block[itra]))
	{
	  smash[itra] = tolower (block[itra]);
	}
      if (islower (block[itra]))
	{
	  smash[itra] = block[itra];
	}
      else
	{
	  smash[itra] = block[itra];
	}
    }
  free (block);
  for (itra = 0; itra < 26; itra++)
    a26[itra] = 0;
  for (itra = 0; itra < strlen (smash); itra++)
    if (isalpha (smash[itra]))
      for (itrb = 0; itrb < strlen (lclatin); itrb++)
	if (smash[itra] == lclatin[itrb])
	  {
	    a26[itrb] += 1;
	    break;
	  }
  for (ptr = 0, itrd = 0; ptr < strlen (smash); ptr++)
    {
      if (isalpha (smash[ptr]))
	{
	  for (itra = 0, tmp = a26[0]; itra < 26; itra++)
	    {
	      if (tmp < a26[itra])
		{
		  tmp = a26[itra];
		  curs = itra;
		}
	    }
	  a26[curs] = 0;
	  if (tmp > 1)
	    {
	      for (itrb = 0; itrb < strlen (smash); itrb++)
		if (smash[itrb] == lclatin[curs])
		  smash[itrb] = fen[itrd];
	      itrd++;
	    }
	}
    }

  return smash;
}

int 
permute(char *str, char *dbfile)
{

	static int limit=0;
	int tmp;
	static int perm=0;
	unsigned int used[strlen(str)];
	int itra, cnt, ent=0;
	char *spr, *cmp;
	static FILE *fp=NULL;
	static int tdone=0;
	
	unsigned int *stock, stockrnr;

	spr=(char *) calloc(strlen(str), sizeof(char));
	cmp=(char *) calloc(strlen(str), sizeof(char));

	
	/* initialize semaphores */

	if(!limit) {
		limit = strlen(str);
	}
	
	for(itra=0; itra<strlen(str); itra++)
		used[itra]=0;

	/* get number of permutations */

	if(!perm)
	{
	   for(itra=strlen(str), perm=1; itra>0; itra--)
	     perm=perm*itra;
	}

	/* calculate all permutations */
	stock=(unsigned int *) calloc(limit, sizeof(unsigned int));
	for(cnt=0, stockrnr=0;cnt!=strlen(str);stockrnr++)
	{
		
		/* Yeh, it'll be faster now... */		
		tmp=rselect(limit, stock, stockrnr);
		stock[stockrnr]=tmp;

		/*
		 	Question is why? Well... We could do and do the Pn permutations
			but I don't get the feeling it's all that general.
		*/		
		if(!used[tmp]) {
			used[tmp]++;
			spr[cnt]=str[tmp];
			if(cnt<strlen(str))
				cnt++;
		}
		usleep(100);

	}
	
	/* print and redo */

	if(fp==NULL && (fp=fopen(dbfile, "a+"))==NULL)
		exit(0);
	
	if(tdone++<perm)
	{
		rewind(fp);
		while(!feof(fp))
		{
			fscanf(fp, "%s", cmp);
			if(!strcmp(spr,cmp)) 
				ent++;
		}
		if(!ent)
		{
			fprintf(fp, "%s\n", spr);
			
			/* display */

			fprintf(stdout, "%s\n", spr);
		}
		if(ent)
			tdone--;
		
		/* recurre */

		permute(str, dbfile);
	}
	fclose(fp);
	return 0;
}

int combine(char *str, int cell, char *dbfile)
{

  FILE *fp;
  int used[strlen(str)];
  int itra, cnt, ent=0;
  int limit, tmp;
  int perma, permb, permc;
  static int comb=0;
  char *spr, *cmp;
  static int tdone=0;
  static int *stock, stockrnr;

  spr = (char *) calloc(cell, sizeof(char));
  cmp = (char *) calloc(cell, sizeof(char));

  /* initialize */

  for(itra=strlen(str); itra>=0; itra--)
    used[itra]=0;

  /* calculate number of valid combinations */

  if(!comb)
  {
    for(itra=strlen(str), perma=1; itra>0; itra--)
      perma=perma*itra;
  
    for(itra=strlen(str)-cell, permb=1; itra>0; itra--)
      permb=permb*itra;

    for(itra=cell, permc=1; itra>0; itra--)
      permc=permc*itra;

    comb=perma/(permb*permc);

    if(comb==1)
      {
        permute(str, dbfile);
        exit(0);
      }
  }

  limit = strlen(str);

  stock=(unsigned int *) calloc(limit, sizeof(unsigned int));
  for(cnt=0, stockrnr=0; cnt!=cell; stockrnr++)
    {	    
		
	/* Same goes here. It's called KARMA! */		
	tmp=rselect(limit, stock, stockrnr);
	stock[stockrnr]=tmp;

      if(!used[tmp]) {
        used[tmp]++;
        spr[cnt]=str[tmp];
        if(cnt<cell)
          cnt++;
      }
      usleep(100);
    }

  if(tdone++<comb)
  {
    if((fp=fopen(dbfile, "r"))!=NULL)
      {
        while(!feof(fp))
          {
            fscanf(fp, "%s\n", cmp);
            cmp[strlen(spr)]='\0';
            if(!strcmp(spr, cmp))
              ent++;
          }
        fclose(fp);
      }
    if(!ent && (fp=fopen(dbfile, "a"))!=NULL)
      {
         fprintf(fp, "%s\n", spr);

          /* display */

          fprintf(stdout, "%s\n", spr);

          fclose(fp);
       }
    if(ent)
      tdone--;

    /* recurre */

    combine(str, cell, dbfile);

  }

  return 0;
}

/* Descrambler, called scrabble... uh... whatever. */

int scrabble(char *str, char *dict) {
	
	FILE *fp;
	int char_slots, *str_stats, *cmp_stats;
	int itra, itrb, sem;
	char *cmp, c;

	str_stats=(int *) calloc(26, sizeof(int));
	cmp_stats=(int *) calloc(26, sizeof(int));
	
	/* get slots and build stats*/
	for(itra=0; itra<26; itra++)
		str_stats[itra]=0;

	for(itra=0; itra<strlen(str); itra++)
		{
			str_stats[str[itra]-97]++;
		}
	
	/* show me the words */
	if((fp=fopen(dict, "r"))==NULL)
		exit(0);
	
	/* what's your longest dict size */
	itra=itrb=0;
	while((c=fgetc(fp))!=EOF)
		{
			if(c=='\n')
			{
				if(itra>itrb) {
					itrb=itra;
				}
				itra=0;
			}
			else
			{
				itra++;
			}
		}
	fclose(fp);
	
	/* ok, let's allocate */
	cmp = (char *) calloc(itrb, sizeof(char));
	
	/* now we can run */
	
	if((fp=fopen(dict, "r"))==NULL)
		exit(0);
		
	while(!feof(fp))
	{
		cmp = (char *) calloc(itrb, sizeof(char));
		fscanf(fp, "%s\n", cmp);
		for(itra=0; itra<26; itra++)
			cmp_stats[itra]=0;
		
		for(itra=0; itra<strlen(cmp); itra++)
			cmp_stats[cmp[itra]-97]++;

		/*compare*/
		for(itra=0, sem=0; itra<26; itra++)
			if(cmp_stats[itra]!=str_stats[itra])
				sem++;
			
		if(!sem)
			{
				fprintf(stdout, "%s\n", cmp);
			}

	}
	
	fclose(fp);
	return 0;
	
}

/* Hill Climbing technique. */

int 
hill_climb(char *org, char *key, char cypher, int pool)
{

  double ENIC=0.0667;
  float freq_l[26], freq_c[26], freq_r[26];
  double score_l, score_c, score_r, score_f;
  static double *voltage;
  double cmp;
  static int runner=0;
  int ratio;
  int sema, semb;

  static int travel=0;
  static int cycles=0;

  int itra;
  char *key_l, *key_r, *key_f;
  char *crypt_l, *crypt_c, *crypt_r;
  static char **key_found;
  static char *key_run;

  int carlo[strlen(key)];

  static int keysize=sizeof(char);

  char *(*cp)(char *org, char *key);
  switch(cypher)
    {
      case 'v':
        cp=vigenere;
        break;
      case 'p':
        cp=playfair;
        break;
      case 'b':
        cp=bifid;
        break;
      case 'l':
        cp=lewis;
        break;
      case 'i':
        cp=alberti;
        break;
      default:
        fault(KEYB, "The selected cypher doesn't support Hill Climbing.");
        break;
    }

  crypt_l=(char *) calloc(strlen(key), sizeof(char));
  crypt_c=(char *) calloc(strlen(key), sizeof(char));
  crypt_r=(char *) calloc(strlen(key), sizeof(char));
  if(key_found==NULL)
    key_found=(char **) calloc(26*sizeof(keysize), sizeof(char));
  if(voltage==NULL)
    voltage=(double *) calloc(26*sizeof(keysize), sizeof(float));

  /* Get frequency for initial key */
  al_dec=true;

  crypt_c=cp(org, key);
  for(itra=0; itra<26; itra++)
    freq_c[itra]=0;
  for(itra=0; itra<strlen(crypt_c); itra++)
    if(isalpha(crypt_c[itra]))
      {
        freq_c[tolower(crypt_c[itra])-97]++;
      }
  for(itra=0, ratio=0; itra<strlen(crypt_c); itra++)
    if(isalpha(crypt_c[itra]))
      ratio++;
  ratio*=(ratio-1);
  for(itra=0, score_c=0; itra<26; itra++)
    score_c=score_c+(freq_c[itra]*(freq_c[itra]-1))/ratio;

  /* Generate new keys */
  key_l=(char *) calloc(strlen(key), sizeof(char));
  key_l=key_mod(key, 1);

  key_r=(char *) calloc(strlen(key), sizeof(char));
  key_r=key_mod(key, 0);

  /* Set decrypt mode */
  al_dec=true;

  /* Generate plaintext for both keys */
  crypt_l=cp(org, key_l);
  crypt_r=cp(org, key_r);

  /* Initialize frequency matrix */
  for(itra=0; itra<26; itra++)
    freq_l[itra]=freq_r[itra]=0;

  /* Get frequency distributions on both texts */
  for(itra=0; itra<strlen(crypt_l); itra++)
    if(isalpha(crypt_l[itra]))
      {
        freq_l[tolower(crypt_l[itra])-97]++;
      }
  for(itra=0; itra<strlen(crypt_r); itra++)
    if(isalpha(crypt_r[itra]))
      {
        freq_r[tolower(crypt_r[itra])-97]++;
      }

  /* Apply scores to both sides */
  for(itra=0, ratio=0; itra<strlen(crypt_l); itra++)
    if(isalpha(crypt_l[itra]))
      ratio++;
  ratio*=(ratio-1);

  for(itra=0, score_l=0; itra<26; itra++)
    score_l=score_l+(freq_l[itra]*(freq_l[itra]-1))/ratio;

  for(itra=0, ratio=0; itra<strlen(crypt_r); itra++)
    if(isalpha(crypt_r[itra]))
      ratio++;
  ratio*=(ratio-1);

  for(itra=0, score_r=0; itra<26; itra++)
    score_r+=(freq_r[itra]*(freq_r[itra]-1))/ratio;

  /* Find closest match to IC */

  key_f=(char *) calloc(strlen(key), sizeof(char));
  if(fabs(score_l-ENIC) <= fabs(score_r-ENIC))
    {
      /* Slide left. */
      if(fabs(score_l-ENIC)<=fabs(score_c-ENIC))
        {
          score_f=score_l;
          strcpy(key_f, key_l);
        }
      else
        {
          score_f=score_c;
          strcpy(key_f, key);
        }
    }
  else
    {
      /* Slide right. */
      if(fabs(score_r-ENIC) < fabs(score_c-ENIC))
        {
          score_f=score_r;
          strcpy(key_f, key_r);
        }
      else
        {
          score_f=score_c;
          strcpy(key_f, key);
        }
    }

  /* Store key for comparison. */
  if(key_run==NULL)
    key_run=(char *) calloc(strlen(key), sizeof(char));

  if(!strcmp(key_run, key_f))
    {

      if(cycles==pool)
      {

        /* Grab closest to IC */
        for(itra=1, cmp=fabs(ENIC-voltage[0]), semb=0; itra<=runner; itra++)
          {
            if(fabs(ENIC-voltage[itra])<cmp)
              {
                cmp=fabs(ENIC-voltage[itra]);
                sema=itra;
              }
             else if(cmp==fabs(ENIC-voltage[itra]))
                semb++;
          }

        printf("Highest score was: %f at index: %d with key: %s and %d possibilities.\n", voltage[sema], sema, key_found[sema], ++semb);
        return 0;
      }

      /* Peak found, store it. */;
      voltage[runner]=score_f;
      key_found[runner]=(char *) calloc(strlen(key), sizeof(char));
      strcpy(key_found[runner],key_f);
      runner++;

      /* Generate jump coordinates. */
      for(itra=0; itra<strlen(key); itra++)
          carlo[itra]=rand()%25;
      key_f=(char *) calloc(strlen(key), sizeof(char));
      for(itra=0; itra<strlen(key); itra++)
        key_f[itra]=65+carlo[itra];
      travel=0;
      cycles++;
    }
  else
    {
      strcpy(key_run, key_f);
      travel++;
    }

  hill_climb(org, key_f, cypher, pool);

  return 1;

}

char *key_mod(char *key, int down)
{

  char *aux;

  int itra;
  int sema;
  int carrier;

  int pol[strlen(key)];
  int adv[strlen(key)];
  int res[strlen(key)];

  for(itra=0; itra<strlen(key); itra++)
    if(itra==strlen(key)-1)
      adv[itra]=1;
    else
      adv[itra]=0;

  for(itra=0, sema=0; itra<strlen(key); itra++)
    if(down)
      {
        if(key[itra]=='A')
          pol[itra]=1;
        else
          pol[itra]=0;
      }
    else
      {
        if(key[itra]=='Z')
          pol[itra]=1;
        else
          pol[itra]=0;
      }

  for(itra=strlen(key)-1, carrier=0; itra>=0; itra--)
    {
      if(adv[itra]+pol[itra]>1)
        carrier=1;
      if(adv[itra]+pol[itra]+carrier>1)
        {
          res[itra]=0;
        }
      else if(adv[itra]==0 && carrier!=1)
        {
          res[itra]=0;
        }
      else
        {
          res[itra]=adv[itra]+pol[itra]+carrier;
          carrier=0;
        }
    }

  aux=(char *) calloc(strlen(key), sizeof(char));
  strcpy(aux, key);
  for(itra=0, sema=0; itra<strlen(key); itra++)
    {
      if(down)
        {
          if(res[itra]==1)
            {
              aux[itra]--;
              sema++;
            }
          if(sema==1 && res[itra]==0)
            aux[itra]='Z';
        }
      else
        {
          if(res[itra]==1)
            {
              aux[itra]++;
              sema++;
            }
          if(sema==1 && res[itra]==0)
            aux[itra]='A';
        }
    }

  return aux;

}


