/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
	      programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

#include "alphCRYPT.h"

char 
*lewis(char *org, char *key)
{

	int itra, itrb, itrc;
	char *hlpr;
	char *dst;
	char *bkp;
	char jeff[27][26] = {		{'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&' },
							{'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a' },
							{'d', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b' },
							{'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c' },
							{'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd' },
							{'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e'},
							{'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f'},
							{'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g'},
							{'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'},
							{'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i'},
							{'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j'},
							{'m', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k'},
							{'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l'},
							{'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm'},
							{'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n'},
							{'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o'},
							{'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'},
							{'s', 't', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q'},
							{'t', 'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r'},
							{'u', 'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's'},
							{'v', 'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't'},
							{'w', 'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u'},
							{'x', 'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v'},
							{'y', 'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w'},
							{'z', '&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x'},
							{'&', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y'},
							{'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z' }
					};

	hlpr=(char *) calloc(strlen(org), sizeof(char));
	dst=(char *) calloc(strlen(org), sizeof(char));
	bkp=(char *) calloc(strlen(org), sizeof(char));

	for(itra=0, itrb=0; itra<strlen(org); itra++)
		if(isalpha(org[itra]) || org[itra]=='&')
			bkp[itrb++]=tolower(org[itra]);

	for(itra=0, itrb=0; itra<strlen(bkp); itra++)
	{
		if(itrb==strlen(key))
			itrb=0;
		hlpr[itra]=tolower(key[itrb++]);
	}

	if(!al_dec)
		for(itra=0, itrb=0; itra<strlen(bkp); itra++)
			dst[itrb++]=jeff[ bkp[itra] - 97 ][ hlpr[itra] - 97 ];
	else
	{
		for(itra=0, itrc=0; itra<strlen(bkp); itra++)
			for(itrb=0; itrb<27; itrb++)
				if(bkp[itra]==jeff[itrb][hlpr[itra]-97])
					dst[itrc++]=itrb+97;
	}

	bkp=(char *) calloc(strlen(org), sizeof(char));
	for(itra=0, itrb=0; itra<strlen(org); itra++)
		if(! (isalpha(org[itra]) || org[itra]=='&'))
			bkp[itra]=org[itra];
		else
			if(isupper(org[itra]))
				bkp[itra]=toupper(dst[itrb++]);
			else
				bkp[itra]=dst[itrb++];


	return bkp;
												
}
