/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
	      programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

/* gd image library request */
#include <gd.h>

/* project includes. */
#include "alphCRYPT.h"

char  *millenium_dec(char *in, char *key)
{
	
	gdImagePtr im_in, im_key;
	FILE *infile, *inkey;
	int in_size_x, in_size_y, key_size_x, key_size_y;
	int pxl_in, pxl_key;
	int diff_r, diff_g, diff_b;
	int itra, itrb, itrc;
	
	char *plaintext;
	
	/* Open the two files side by side. */
	if((infile=fopen(in, "rb"))==NULL)
		fault(KEYB, "The keyfile couldn't be opened for reading.");
	
	if((inkey=fopen(key,"rb"))==NULL)
		fault(KEYB, "The keyfile couldn't be opened for reading.");
	
	/* Create images from both of them. */
	im_in = gdImageCreateFromPng(infile);
	im_key = gdImageCreateFromPng(inkey);
	
	/* Sanity check the two files. */
	in_size_x = gdImageSX(im_in);
	in_size_y = gdImageSY(im_in);
	key_size_x = gdImageSX(im_key);
	key_size_y = gdImageSY(im_key);	
	
	if(in_size_x+in_size_y != key_size_x+key_size_y)
		fault(KEYB, "The keyfile and ecryption file don't match.");
	
	plaintext = (char *) calloc(in_size_x*in_size_y, sizeof(char));
	
	for(itra=0, itrc=0; itra<in_size_x; itra++)
		for(itrb=0; itrb<in_size_y; itrb++)
		{
			pxl_in=gdImageGetPixel(im_in, itra, itrb);
			pxl_key=gdImageGetPixel(im_key, itra, itrb);
			
			diff_r = gdImageRed(im_in, pxl_in) - gdImageRed(im_key, pxl_key);
			diff_g = gdImageGreen(im_in, pxl_in) - gdImageGreen(im_key, pxl_key);
			diff_b = gdImageBlue(im_in, pxl_in) - gdImageBlue(im_key, pxl_key);
			
			if(diff_r != 0 || diff_g != 0 || diff_b !=0) {
				if(diff_r<0)
					diff_r=diff_r*-1;
				if(diff_g<0)
					diff_g=diff_g*-1;
				if(diff_b<0)
					diff_b=diff_b*-1;
				plaintext[itrc++]=diff_r+diff_g+diff_b;
			}
		}
		return plaintext;
		
}


int millenium_enc(char *org, char *keyfile, char *dst)
{
	
	gdImagePtr im;
	FILE *inkey, *out;
	int id;
	int itra, itrb, itrc, itrd;
	int size_x, size_y, total_pixels, pad_pixels;
	int red, green, blue, pixel_mod, pxl, *plex[3];
	
	/* Determine the image type. */
	if((inkey=fopen(keyfile, "rb"))==NULL)
		fault(KEYB, "The keyfile couldn't be opened for reading.");
	for(itra=0, id=0; itra<3; itra++)
		id+=fgetc(inkey);
	rewind(inkey);

	/* Check for PNG image. */
	if(id==295)
		im=gdImageCreateFromPng(inkey);
	else {
		fclose(inkey);
		fault(KEYB, "The supplied keyfile should be a PNG image.");
	}
	
	fclose(inkey);
	
	/* Get total image size. */
	size_x=gdImageSX(im);
	size_y=gdImageSY(im);
	
	/* Make sure message would fit. */
	total_pixels = size_x * size_y;
	if(strlen(org)>total_pixels)
		fault(ENTO, "The message to hide is too long.");
	
	/* Get the pad. */
	pad_pixels=total_pixels/strlen(org);
	
	/* Allocate the plex matrix. */
	plex[0] = calloc(strlen(org), sizeof(int));
	plex[1] = calloc(strlen(org), sizeof(int));
	plex[2] = calloc(strlen(org), sizeof(int));
	
	for(itra=0; itra<strlen(org); itra++) {
		plex[0][itra] = org[itra]/3;
		plex[1][itra] = org[itra]/3;
		plex[2][itra] = org[itra]-2*plex[1][itra];
	}
	
	for(itra=0, itrc=0, itrd=0; itra<size_x; itra++)
		for(itrb=0; itrb<size_y; itrb++, itrc++)
			{
				if(itrc%pad_pixels==0 && itrd<strlen(org)) {
					
					pxl=gdImageGetPixel(im, itra, itrb);
					
					red=gdImageRed(im, pxl);
					green=gdImageGreen(im, pxl);
					blue=gdImageBlue(im, pxl);
					

					if(red+plex[0][itrd]>255)
						red-=plex[0][itrd];
					else
						red+=plex[0][itrd];
					
					if(green+plex[1][itrd]>255)
						green-=plex[1][itrd];
					else
						green+=plex[1][itrd];
					
					if(blue+plex[2][itrd]>255)
						blue-=plex[2][itrd];
					else
						blue+=plex[2][itrd];
					
					pixel_mod=gdImageColorAllocate(im, red, green, blue);
					
					gdImageSetPixel(im, itra, itrb, pixel_mod);
					
					itrd++;
				}
			}
			
	/* Create encrypted file. */
	if((out=fopen(dst, "wb"))==NULL)
		fault(KEYB, "Couldn't write the encrypted file to current directory.");
	gdImagePng(im, out);
	fclose(out);
	gdImageDestroy(im);
	
}

char *millenium(char *org, char *src, char *dst)
{
	
	char *mess;
	
	if(strlen(org) && !al_dec)
	{
		millenium_enc(org, src, dst);
		return "File written successfully.\n";
	}
	else if(al_dec)
	{
		mess=millenium_dec(src, dst);
		return mess;
	}
		
}
